# this code estimates confidence sets via bootstrap
# it calls est_pars(), in est_pars.R, for each of nSamp bootstrap samples and records the estimates in a dataframe
#   note: est_pars.R also contains the code to run the regressions presented in Tables 2, 3, 4
# it then runs calc_conf_sets_given_scenario.R, which uses the estimates to bootstrap confidence intervals for delta_h and delta_I


rm(list=ls())

library(foreign)
library(stargazer)
library(MASS)
library(tikzDevice)
library(collapse)
library(ggplot2)
library(ggthemes)
library(doParallel)
library(doRNG)
library(dplyr)

cl_size<-detectCores()
simulate_bootstrap<-FALSE
serial_est_conf_set<-FALSE
nSamp<-1e3        # number of bootstrap iterations (1e5 in paper)
size_asym_alpha_grid<-500
size_alpha<-0.05 # (symmetric-)tail width of larger confidence interval (smaller confidence interval will have half this tail probability). note: tails are not assumed to be symmetric. 
underline_delta<-0
overline_delta<-1
beta_h<-1
xpd_val<-FALSE
## plotting parameters
point_id_bump <- 0.0005

# values for robustness checks
## note: need to re-run for both values of alt_Delta_l_1! 
alt_beta_y <- 0.2
#alt_Delta_l_1 <- 0.0
#\cite[p.~2072]{attanasio2018aggregating} report a median (Marshallian) labor supply elasticity of 0.18. 
#At the sample mean of 0.833, this value would imply that a 3\% increase in wage (Glewwe et al say prize ranged from less than 2\% to 4\% of annual salary...) would increase labor supply (hence, reduce leisure) by 0.0045.
alt_Delta_l_1 <- -0.0045

# read in initial data
glewwe<-read.dta("./data/merged_for_R.dta")
glewwe<-glewwe[with(glewwe, order(pupid, year, j, k, na.last=T)),]
glewwe$teso<-glewwe$d==1 | glewwe$d==2
glewwe<-subset(glewwe, teso==0)
## variables
# pupid: student (panel ID) variable
# year: calendar year. 1997: no treatment, 1998-99: two years of treatment, 2000: some students had been treated but no contemporaneous treatment
# treatgrp: indicator for being in the treatment group that year (i.e., zero for all in 1997 but around 1/2 of students have 1 in 1998-2000)
# j: grade
# t: govt test, per subject k
# k: subject 
# * k=1 English
# * k=2 Math
# * k=3 Science
# * k=4 Swahili
# * k=5 Geog./Hist./Chrit.
# * k=6 Art/Crafts/Music
# * k=7 Home Sic./Business
# ntch: total teachers
# htch: headteachers
# dhtch: mid-level teachers?
# astch: youngest teachers
# male: number of male teachers
# sexdum seems to be indicator for student is male (or female)
glewwe$absent_school<- 1-glewwe$attend
glewwe$absent_classroom<- 1-glewwe$present

source("est_pars.R")

# create these to sample glewwe, stratifying by treat
glewwe_treated<-subset(glewwe, treated==1)
glewwe_control<-subset(glewwe, treated==0)

if (simulate_bootstrap==TRUE){

  cl<-makeCluster(cl_size)
	registerDoParallel(cl)
	registerDoRNG(seed=3285)
	est_df<-foreach(ss=1:nSamp, .combine = 'rbind') %dopar% {
			this_treated<-glewwe_treated[sample(1:dim(glewwe_treated)[1], replace=TRUE),]
			this_control<-glewwe_control[sample(1:dim(glewwe_control)[1], replace=TRUE),]
			this_data<-rbind(this_treated, this_control)
			this_est<-est_pars_samp(this_data, samp=ss)
			this_est$est_hat			
			}
	stopCluster(cl)

	write.csv(est_df, file=paste0("./output/est_df_", nSamp, ".csv"), row.names=FALSE)
} else {
	est_df<-read.csv(paste0("./output/est_df_", nSamp, ".csv"))
}

# describe the data
pdf("./output/plot_gamma_hat.pdf")
	par(mfrow = c(4,1))
	for (dd in 1:4){
		hist(est_df[,dd], main = names(est_df)[dd])
	}
	par(mfrow = c(1,1))
	plot(est_df, pch = ".", col = rgb(red = 0, green = 0, blue = 0, alpha = 0.5))
dev.off()

print(apply(est_df, 2, mean))
print(apply(est_df, 2, sd))

## form CIs

# from \ref{eq:delta_h_fxn_of_delta_I}
delta_h_given_stuff<-function(delta_I) function(Delta_y_1, Delta_y_2, Delta_l_1, beta_y){
  beta_h*(Delta_y_2/Delta_y_1 - beta_y) + delta_I*(1+beta_h*Delta_l_1)
}

delta_I_given_stuff<-function(delta_h) function(Delta_y_1, Delta_y_2, Delta_l_1, beta_y){
  (delta_h-beta_h*(Delta_y_2/Delta_y_1 - beta_y))/(1+beta_h*Delta_l_1)
}

pi_intercept<-function(Delta_y_1, Delta_y_2, Delta_l_1, beta_y){
  est<-beta_h*(Delta_y_2/Delta_y_1 - beta_y)
  est
}

pi_slope<-function(Delta_y_1, Delta_y_2, Delta_l_1, beta_y){
  est<- 1 + beta_h*Delta_l_1/Delta_y_1
  est
}

##########
### form marginal confidence intervals
### 1. estimate bounds
### 2. estimate confidence sets
##########

## simulate locus 
## depends on scenario
scenario <- "baseline"

  pi_intercept_sim<-with(est_df, Vectorize(pi_intercept)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=Delta_l_1_sim, beta_y=beta_y_sim))
  pi_slope_sim<-with(est_df, Vectorize(pi_slope)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=Delta_l_1_sim, beta_y=beta_y_sim))
  output_path <- "baseline"
  if (!dir.exists(paste0("./output/", output_path))) {dir.create(paste0("./output/", output_path))}
  source("calc_conf_sets_given_scenario.R")

scenario <- "robustness_beta_y"

  pi_intercept_sim<-with(est_df, Vectorize(pi_intercept)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=Delta_l_1_sim, beta_y=rep(alt_beta_y, nSamp)))
  pi_slope_sim<-with(est_df, Vectorize(pi_slope)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=Delta_l_1_sim, beta_y=rep(alt_beta_y, nSamp)))
  output_path <- paste(scenario, alt_beta_y, sep = "_")
  if (!dir.exists(paste0("./output/", output_path))) {dir.create(paste0("./output/", output_path))}
  source("calc_conf_sets_given_scenario.R")

scenario <- "robustness_Delta_l_1"
  
  pi_intercept_sim<-with(est_df, Vectorize(pi_intercept)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=rep(alt_Delta_l_1, nSamp), beta_y=beta_y_sim))
  pi_slope_sim<-with(est_df, Vectorize(pi_slope)(Delta_y_1=Delta_y_1_sim, Delta_y_2=Delta_y_2_sim, Delta_l_1=rep(alt_Delta_l_1, nSamp), beta_y=beta_y_sim))
  output_path <- paste(scenario, alt_Delta_l_1, sep = "_")
  if (!dir.exists(paste0("./output/", output_path))) {dir.create(paste0("./output/", output_path))}
  source("calc_conf_sets_given_scenario.R")
  